/******************* 3DoT Configure ****************
   6/9/2022 by Gary Hill
   File compatible with 3DoT v4.45 to v12 schematics
   Path: Arduino > libraries > ArxRobot_Libary > src
   Leonardo variant #define macros and static const definitions in 
   C:\Program Files (x86)\Arduino\hardware\arduino\avr\variants\leonardo
***************************************************/
#ifndef Configure_h
#define Configure_h

/******************* Debug Mode ********************
 * When debugging your program set DEBUG and ECHO_COMMANDS to TRUE. Once your code is bug free,
 * to avoid latency errors and minimize bandwidth set to FALSE (the default value).
 ***************************************************/
#define FALSE  0
#define TRUE   1
#define DEBUG  FALSE
#define ECHO_COMMANDS  FALSE

/******************* Blink, TX LED, and Program LED **********
 * The Arduino Leonardo 32U4 Blink LED is on digital pin PC7 pin 13. 
 * Arduino reserved word LED_BUILTIN maps to the Blink LED. 
 * 3DoT v5.03 to 9.1b Texas also used PC7 pin 13 as the Blink LED and 
 * to indicate board was in program mode. 3DoT v10.1 and later used 
 * PD5 pin 30 as the Blink LED and to indicate board was in program mode.
 * 
 * In addition to the Blink LED, programmers can write to the TX LED wired to the HM-11.
 * Legacy versions of the board used pin GPIO PD5 / Arduino D30 for this LED. 
 * 3DoT 9.1b Texas and later simply combined the TX LED with the 32U4 PD2(TX)/D1 pin.
 ***************************************************/

#if defined(ARDUINO_AVR_3DOT_LEGACY)
  #define LED 30         // TX indicator LED wired to PD5 / Arduino D30
  #define LED_BUILTIN 13 // Arduino default constant, plus 3DoT Program mode
#elif defined(ARDUINO_AVR_3DOT_V9_TEXAS)
  #define LED 1          // TX indicator LED wired to PD2(TX)/D1 pin
  #define LED_BUILTIN 13 // Arduino default plus 3DoT Program mode
#else
  #define LED 1          // TX indicator LED wired to PD2(TX)/D1 pin
  #define LED_BUILTIN 30 // Blink and Program LED wired to PD5 / Arduino D30
#endif

/******************* Motor Driver ********************
 * 3DoT version 8.01_KISS and earlier used a Toshiba TB6612 Motor Driver.
 * 3DoT version 9.10 and later used the TI DRV8848 Motor Driver.
 * 3DoT version 10+ is the new default pinout for 32U4 to DRV8848 interface. 
 * 
 * A  = left motor (simply a convention)
 *****************************************************/
#if defined(ARDUINO_AVR_3DOT_LEGACY)
  /* Mapping for 6.43a [source: 3DoT6_43a.sch] 
     boards using TB6621FNG motor driver */
  #define STBY   8  // standby  (Sparkfun ProMicro Analog Pin A0)
  #define NSLEEP 8  // default 10+ naming convention   
  //Motor A   left motor
  #define PWMA   6  // Speed control
  #define AIN1  12  // Direction
  #define AIN2   4  // Direction  
  //Motor B   right motor  
  #define PWMB  10  // Speed control
  #define BIN1   9  // Direction (Analog Pin A1)
  #define BIN2   5  // Direction (Analog Pin A2)

#elif defined(ARDUINO_AVR_3DOT_V9_TEXAS)
  /* Mapping for 9.10b "Texas" [source: 3DoT9_1b_Texas.sch]
     First generation of boards using DRV8848 motor driver */
  #define AIN1 6                   // Motor A
  #define AIN2 9
  #define NSLEEP 4
  #define BIN1 5                   // Motor B 
  #define BIN2 10
  #define STBY 4                   // legacy naming convention

#else  // v10+
  /* Mapping for 10 */
  #define AIN1   11   // PB7         Motor A
  #define AIN2    6   // PD7, OC4D
  #define NSLEEP  8   // PB4        // NC on version 10 (see mapping at end of file)
  #define BIN1   13   // PC7, OC4A   Motor B
  #define BIN2   10   // PB6, OC4B
  #define STBY    8                 // legacy naming convention
#endif

/********************** Servos ***********************
 * 3DoT version 9.1b_Texas used older standard 
 * 3DoT version 10.1 (Altium Designs) and later use new default standard
 *****************************************************/

#if defined(ARDUINO_AVR_3DOT_LEGACY) || defined(ARDUINO_AVR_3DOT_V9_TEXAS)
  #define SERVO_A  7
  #define SERVO_B 11
#else
  #define SERVO_A  4  
  #define SERVO_B 12
#endif

const int VBATT_PIN = A5; // A5 is an Arduino reserved word
const int I2C_SDA = 2;    // 3.3v I2C
const int I2C_SDL = 3;

/******************* Packet IDs ********************
   source: Arxterra
 ***************************************************/
#define COMMAND_PACKET_ID    0xA5
#define TELEMETRY_PACKET_ID  0xCA

/******************* Robot Commands ****************
   source: Arxterra Technical Folder / Communication between Arduino
           Phone, and Control Panel - Phone to Arduino (starting at line 57)
 ***************************************************/
// Arxterra Commands to Numeric Value Mapping
//               Data[0] =           CMD TYPE | Qual | Arguments
//                                     7-bits  1-bit   Bytes (note: N = 1 + bytes)
//                              bit => 7654321   0
#define MOVE                 0x01   // 0000000   1     4
#define CAMERA_MOVE          0x02   // 0000001   0     4
#define CAMERA_MOVE_HOME     0x04   // 0000010   0     0
#define CAMERA_MOVE_RESET    0x05   // 0000010   1     0
#define READ_EEPROM          0x06   // 0000011   0     3
#define WRITE_EEPROM         0x07   // 0000011   1     3 + b
#define SAFE_ROVER           0x08   // 0000100   0     0
#define SLEEP                0x0A   // 0000101   0     0
#define WAKEUP               0x0B   // 0000101   1     0
#define HEADLIGHT_OFF        0x0C   // 0000110   0     0
#define HEADLIGHT_ON         0x0D   // 0000110   1     0
#define COMM_SETUP           0x10   // 0001000   0     1   => see table following (pre-implementation 7/28/2016 PING_INTERVAL allowed for 4 byte argument)
#define PING                 0x11   // 0001000   1     0
#define HEADING              0x12   // 0001001   0     2
#define CURRENT_COORD        0x13   // 0001001   1     see note [1]
#define WAYPOINT_COORD       0x14   // 0001010   0     see note [1]
#define WAYPOINTS_OFF        0x16   // 0001011   0     0
#define WAYPOINTS_ON         0x17   // 0001011   1     0
#define WAYPOINTS_CLEAR_ALL  0x18   // 0001100   0     0
#define WAYPOINT_MOVE        0x19   // 0001100   1     9
#define WAYPOINT_DELETE      0x1A   // 0001101   0     1
#define WAYPOINT_VIEW_CLICK  0x1B   // 0001101   1     4

// note(s)
// [1] See Waypoint Coordinates class in "Communication between Arduino,
//     Phone, and Control Panel" Google Drive Spreadsheet.

// 0x40 - 0x5F  32 Custom Commands, 1 - 2 byte arguments

/********************** 0x10 WATCHDOG SETUP *****************
   byte
   0x00  Watchdog Off
   0x4E  1 sec interrupt and system reset mode
   0x4F  2 sec
   0x68  4 sec
   0x69  8 sec
   0x46  1 sec interrupt mode
   0x47  2 sec
   0x60  4 sec
   0x61  8 sec
*************************************************************/

/******************* Robot Telemetry ****************
   source: Arxterra Technical Folder / Communication between Arduino
           Phone, and Control Panel - Arduino to Phone (starting at line 84)
           see reference document for argument (typically assume int (i.e., 16 bit signed)
 ***************************************************/
// Telemetry Identifiers to Numeric Value Mapping
const uint8_t MOTOR1_CURRENT_ID  = 0x01;  // motor 1 is left motor
const uint8_t MOTOR2_CURRENT_ID  = 0x02;  // motor 2 is right motor
const uint8_t TEMP_SENSOR_ID     = 0x03;  // temperature sensor
const uint8_t RANGE_LEFT_ID      = 0x04;  // ultrasonic range 1 is left
const uint8_t RANGE_RIGHT_ID     = 0x05;  // ultrasonic range 2 is right
const uint8_t BATTERY_ID         = 0x06;  // Arduino battery
const uint8_t DIRTY_BATTERY_ID   = 0x07;  // DC and servo motors battery
const uint8_t PAN_POSITION_ID    = 0x08;  // originally defined as pan and tilt
const uint8_t TILT_POSITION_ID   = 0x09;  // not in original definition
const uint8_t EEPROM_RESPONSE_ID = 0x0A;  // sent in response to EEPROM Read Command
const uint8_t EMERGENCY_ID       = 0x0B;  // arduino emergency code (see subtable below)

const uint8_t COMMAND_ECHO_ID    = 0x0D;
const uint8_t EXCEPTION_ID       = 0x0E;  // arduino error code exception (see subtable below)
const uint8_t PONG_ID            = 0x11;
const uint8_t ROUTE_STATUS_ID    = 0x12;
const uint8_t WAYPOINT_ARRIVE_ID = 0x13;

/********************** 0x0E Exception Codes *****************
   high   low order byte
    01    Start byte 0xA5 expected
    02    Packet length out of range 1 - 20
    03    LRC checksum error
    04    Undefined command decoder FSM state
    05    Array out of range i >= 23
    06    Watchdog timeout out of range
*************************************************************/

/********************** 0x0B Emergency Codes *****************
    high   low order byte
    01     00               Watchdog timeout
*************************************************************/
const uint16_t WATCHDOG_TIMEOUT = 0x0100;

#endif

/********************** Board Configurations *****************
 *  Header file configures pins TXLED to STBY. HAT and Sensor Shield pin assignments included for reference only. 
 *  Series 4 and earlier 3DoT boards are not supported.
 *  ARDUINO_AVR_3DOT_LEGACY supports Series 5, 6, 7, and 8 3DoT Boards. Series 7 and later use standardized HAT and Sensor shields. 
 *  ARDUINO_AVR_3DOT_V9_TEXAS supports version 9.1b
 *  Default supports versions 10, 11, and 12. Version 10 NSLEEP is wired to 5v_VM, its assigned pin PB4/D8 is not connected (NC) on this board. 
 *  Configuration control notes in early versions of the schematics show that v1 boards once existed in the wild.
 *  
 *             TXLED     PROG/BLINK SERVO               AIN1     AIN2      BIN1    BIN2     STBY     HAT SHIELD                           SENSOR SHIELD                  NOTE
 *                                   A         B                                            / NSLEEP
 *  4.45 [1]    PD5/D10   none       PB7/D11#  PC7/D13#  PC6/D5#  PB6/D10#  PF6/A1  PF5/A2   PF7/A0   SCL, SDA, GND, 3.3V                  none                          May 7, 2016 email Nicholas Lombardo
 *  4.46 [2]    NO CHANGE                                NO CHANGE                                    NO CHANGE                            none
 *  4.52_OSH[0] PD5/D30   PE6/D7     PB7/D11#  PC7/D13#  NO CHANGE                                    A3,A4,SCL,SDA,GND,3.3V               none    
 *  4.53 [3]    NO CHANGE                                NO CHANGE                                    A3,A4,GND,3.3V,SCL,SDA               A6, A10, GND, 3.3V
 *  5.03 [4]    PD5/D30   PC7/D13#   PB7/D11#  PE6/D7    PD6/D12  PD4/D4    PB5/D9# PC6/D5#  PB4/D8   SCL,SDA,3.3V,GND,GND,RESET,A0,A1     SCL,SDA,3.3V,GND,A2,A3                  
 *  --- ARDUINO_AVR_3DOT_LEGACY ---
 *  6.43a [5]   NO CHANGE                                NO CHANGE                                    MISO,MOSI,SCK,SS,AREF,A4,3.3V,VBATT  SCL,SDA,3.3V,GND,A0,A1,A2,A3
 *                                                                                                    RESET,SCL,SDA,RX,TX,GND,GND,VM                                                                                               
 *  7.38 [3]    NO CHANGE                                NO CHANGE                                    MISO,MOSI,SCK,SS,AREF,A4,GND,RAW     NO CHANGE
 *                                                                                                    RESET,SCL,SDA,RX,TX,3.3V,GND,VM        
 *  8.01_KISS [6]                                        NO CHANGE                                    NO CHANGE                            NO CHANGE
 *  9.02 [7]    NO CHANGE                                NO CHANGE                                    NO CHANGE                            NO CHANGE
 *  9.05 [7][8] NO CHANGE                                NO CHANGE                                    NO CHANGE                            NO CHANGE   
 *  --- ARDUINO_AVR_3DOT_V9_TEXAS --- 
 *  9.1b_Texas  PD2(TX)/D1 PC7/D#13 NO CHANGE            PD7/D6#  PB5/D9#   PC6/D5# PB6/D10#  PD4/D4  NO CHANGE                            NO CHANGE                      DRV8848PWPR Motor Driver
 *  --- DEFAULT ---
 *  10   PDF    PD2(TX)/D1 PD5/D30  PD4/D4    PD6/D12    PB7/D11# PD7/D6#   PC7/D13# PB6/D10# 5v_VM [9]                                                                       ALTIUM   
 *  10.1 [0]    NO CHANGE           NO CHANGE            NO CHANGE                            PB4/D8                                                                      w/ jumper wire
 *  12   PDF    NO CHANGE           NO CHANGE            NO CHANGE                            PB4/D8
 *  13
 *  
 *  notes code
 *        name
 *     0. I am not sure if this version was ever fabricated.
 *     1. on/off switch. May be operational with reset button.
 *     2. on/off switch. Tricky to program but possible. 
 *     3. n/a  The single extant version of the v4.53 board does not have a switch.    
 *     4. n/a  Neither of the extant versions of the v5.03 board included a battery holder.    
 *     5. n/a   Of the extant versions of these winged boards (1 v6.43a, 2 v7.38) none are operational.
 *     6. kitty Program LED not available. Blink works. 
 *     7. fox   Configure as a LEGACY 3DoT board.
 *     8. fox   Both white and black PCBs fabricated. Both LED and PROG/BLINK when configured as LEGACY?
 *     9. cat   Version 10 cuts trace to 5v_VM and adds a jumper to GND. Corrected in 10.1 and later by again wiring to PB4/D8
 *************************************************************/
